@@ -83,6 +83,14 @@
  *       </monitor>
  *       <presentation>yes</presentation>
  *     </logicalmonitor>
+ *     <disabled>
+ *       <monitorspec>
+ *         <connector>LVDS3</connector>
+ *         <vendor>Vendor C</vendor>
+ *         <product>Product C</product>
+ *         <serial>Serial C</serial>
+ *       </monitorspec>
+ *     </disabled>
  *   </configuration>
  * </monitors>
  *
@@ -151,7 +159,8 @@ typedef enum
   STATE_MONITOR_MODE_HEIGHT,
   STATE_MONITOR_MODE_RATE,
   STATE_MONITOR_MODE_FLAG,
-  STATE_MONITOR_UNDERSCANNING
+  STATE_MONITOR_UNDERSCANNING,
+  STATE_DISABLED,
 } ParserState;
 
 typedef struct
@@ -159,6 +168,8 @@ typedef struct
   ParserState state;
   MetaMonitorConfigStore *config_store;
 
+  ParserState monitor_spec_parent_state;
+
   gboolean current_was_migrated;
   GList *current_logical_monitor_configs;
   MetaMonitorSpec *current_monitor_spec;
@@ -167,6 +178,7 @@ typedef struct
   MetaMonitorModeSpec *current_monitor_mode_spec;
   MetaMonitorConfig *current_monitor_config;
   MetaLogicalMonitorConfig *current_logical_monitor_config;
+  GList *current_disabled_monitor_specs;
 } ConfigParser;
 
 G_DEFINE_TYPE (MetaMonitorConfigStore, meta_monitor_config_store,
@@ -254,6 +266,10 @@ handle_start_element (GMarkupParseContext  *context,
 
             parser->state = STATE_MIGRATED;
           }
+        else if (g_str_equal (element_name, "disabled"))
+          {
+            parser->state = STATE_DISABLED;
+          }
         else
           {
             g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
@@ -351,7 +367,7 @@ handle_start_element (GMarkupParseContext  *context,
         if (g_str_equal (element_name, "monitorspec"))
           {
             parser->current_monitor_spec = g_new0 (MetaMonitorSpec, 1);
-
+            parser->monitor_spec_parent_state = STATE_MONITOR;
             parser->state = STATE_MONITOR_SPEC;
           }
         else if (g_str_equal (element_name, "mode"))
@@ -456,6 +472,22 @@ handle_start_element (GMarkupParseContext  *context,
                      "Invalid element '%s' under underscanning", element_name);
         return;
       }
+
+    case STATE_DISABLED:
+      {
+        if (!g_str_equal (element_name, "monitorspec"))
+          {
+            g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                         "Invalid element '%s' under disabled", element_name);
+            return;
+          }
+
+        parser->current_monitor_spec = g_new0 (MetaMonitorSpec, 1);
+        parser->monitor_spec_parent_state = STATE_DISABLED;
+        parser->state = STATE_MONITOR_SPEC;
+
+        return;
+      }
     }
 }
 
@@ -513,6 +545,34 @@ derive_logical_monitor_layout (MetaLogicalMonitorConfig    *logical_monitor_conf
   return TRUE;
 }
 
+static void
+finish_monitor_spec (ConfigParser *parser)
+{
+  switch (parser->monitor_spec_parent_state)
+    {
+    case STATE_MONITOR:
+      {
+        parser->current_monitor_config->monitor_spec =
+          parser->current_monitor_spec;
+        parser->current_monitor_spec = NULL;
+
+        return;
+      }
+    case STATE_DISABLED:
+      {
+        parser->current_disabled_monitor_specs =
+          g_list_prepend (parser->current_disabled_monitor_specs,
+                          parser->current_monitor_spec);
+        parser->current_monitor_spec = NULL;
+
+        return;
+      }
+
+    default:
+      g_assert_not_reached ();
+    }
+}
+
 static void
 handle_end_element (GMarkupParseContext  *context,
                     const char           *element_name,
@@ -575,11 +635,9 @@ handle_end_element (GMarkupParseContext  *context,
         if (!meta_verify_monitor_spec (parser->current_monitor_spec, error))
           return;
 
-        parser->current_monitor_config->monitor_spec =
-          parser->current_monitor_spec;
-        parser->current_monitor_spec = NULL;
+        finish_monitor_spec (parser);
 
-        parser->state = STATE_MONITOR;
+        parser->state = parser->monitor_spec_parent_state;
         return;
       }
 
@@ -665,6 +723,14 @@ handle_end_element (GMarkupParseContext  *context,
         return;
       }
 
+    case STATE_DISABLED:
+      {
+        g_assert (g_str_equal (element_name, "disabled"));
+
+        parser->state = STATE_CONFIGURATION;
+        return;
+      }
+
     case STATE_CONFIGURATION:
       {
         MetaMonitorConfigStore *store = parser->config_store;
@@ -701,11 +767,13 @@ handle_end_element (GMarkupParseContext  *context,
           config_flags |= META_MONITORS_CONFIG_FLAG_MIGRATED;
 
         config =
-          meta_monitors_config_new (parser->current_logical_monitor_configs,
-                                    layout_mode,
-                                    config_flags);
+          meta_monitors_config_new_full (parser->current_logical_monitor_configs,
+                                         parser->current_disabled_monitor_specs,
+                                         layout_mode,
+                                         config_flags);
 
         parser->current_logical_monitor_configs = NULL;
+        parser->current_disabled_monitor_specs = NULL;
 
         if (!meta_verify_monitors_config (config, store->monitor_manager,
                                           error))
@@ -849,6 +917,7 @@ handle_text (GMarkupParseContext *context,
     case STATE_MONITOR_SPEC:
     case STATE_MONITOR_MODE:
     case STATE_TRANSFORM:
+    case STATE_DISABLED:
       {
         if (!is_all_whitespace (text, text_len))
           g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
@@ -1058,6 +1127,27 @@ meta_monitor_config_store_lookup (MetaMonitorConfigStore *config_store,
                                                     key));
 }
 
+static void
+append_monitor_spec (GString         *buffer,
+                     MetaMonitorSpec *monitor_spec,
+                     const char      *indentation)
+{
+  g_string_append_printf (buffer, "%s<monitorspec>\n", indentation);
+  g_string_append_printf (buffer, "%s  <connector>%s</connector>\n",
+                          indentation,
+                          monitor_spec->connector);
+  g_string_append_printf (buffer, "%s  <vendor>%s</vendor>\n",
+                          indentation,
+                          monitor_spec->vendor);
+  g_string_append_printf (buffer, "%s  <product>%s</product>\n",
+                          indentation,
+                          monitor_spec->product);
+  g_string_append_printf (buffer, "%s  <serial>%s</serial>\n",
+                          indentation,
+                          monitor_spec->serial);
+  g_string_append_printf (buffer, "%s</monitorspec>\n", indentation);
+}
+
 static void
 append_monitors (GString *buffer,
                  GList   *monitor_configs)
@@ -1073,16 +1163,7 @@ append_monitors (GString *buffer,
                       monitor_config->mode_spec->refresh_rate);
 
       g_string_append (buffer, "      <monitor>\n");
-      g_string_append (buffer, "        <monitorspec>\n");
-      g_string_append_printf (buffer, "          <connector>%s</connector>\n",
-                              monitor_config->monitor_spec->connector);
-      g_string_append_printf (buffer, "          <vendor>%s</vendor>\n",
-                              monitor_config->monitor_spec->vendor);
-      g_string_append_printf (buffer, "          <product>%s</product>\n",
-                              monitor_config->monitor_spec->product);
-      g_string_append_printf (buffer, "          <serial>%s</serial>\n",
-                              monitor_config->monitor_spec->serial);
-      g_string_append (buffer, "        </monitorspec>\n");
+      append_monitor_spec (buffer, monitor_config->monitor_spec, "        ");
       g_string_append (buffer, "        <mode>\n");
       g_string_append_printf (buffer, "          <width>%d</width>\n",
                               monitor_config->mode_spec->width);
@@ -1205,6 +1286,18 @@ generate_config_xml (MetaMonitorConfigStore *config_store)
           append_logical_monitor_xml (buffer, config, logical_monitor_config);
         }
 
+      if (config->disabled_monitor_specs)
+        {
+          g_string_append (buffer, "    <disabled>\n");
+          for (l = config->disabled_monitor_specs; l; l = l->next)
+            {
+              MetaMonitorSpec *monitor_spec = l->data;
+
+              append_monitor_spec (buffer, monitor_spec, "      ");
+            }
+          g_string_append (buffer, "    </disabled>\n");
+        }
+
       g_string_append (buffer, "  </configuration>\n");
     }
 
@@ -1230,9 +1323,10 @@ saved_cb (GObject      *object,
   if (!g_file_replace_contents_finish (G_FILE (object), result, NULL, &error))
     {
       if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-        g_warning ("Saving monitor configuration failed: %s\n", error->message);
-      else
-        g_clear_object (&data->config_store->save_cancellable);
+        {
+          g_warning ("Saving monitor configuration failed: %s\n", error->message);
+          g_clear_object (&data->config_store->save_cancellable);
+        }
 
       g_error_free (error);
     }
@@ -1246,6 +1340,37 @@ saved_cb (GObject      *object,
   g_free (data);
 }
 
+static void
+meta_monitor_config_store_save_sync (MetaMonitorConfigStore *config_store)
+{
+  GError *error = NULL;
+  GFile *file;
+  GString *buffer;
+
+  if (config_store->custom_write_file)
+    file = config_store->custom_write_file;
+  else
+    file = config_store->user_file;
+
+  buffer = generate_config_xml (config_store);
+
+  if (!g_file_replace_contents (file,
+                                buffer->str, buffer->len,
+                                NULL,
+                                FALSE,
+                                G_FILE_CREATE_REPLACE_DESTINATION,
+                                NULL,
+                                NULL,
+                                &error))
+    {
+      g_warning ("Saving monitor configuration failed: %s\n",
+                 error->message);
+      g_error_free (error);
+    }
+
+  g_string_free (buffer, TRUE);
+}
+
 static void
 meta_monitor_config_store_save (MetaMonitorConfigStore *config_store)
 {
@@ -1258,6 +1383,17 @@ meta_monitor_config_store_save (MetaMonitorConfigStore *config_store)
       g_clear_object (&config_store->save_cancellable);
     }
 
+  /*
+   * Custom write file is only ever used by the test suite, and the test suite
+   * will want to have be able to read back the content immediately, so for
+   * custom write files, do the content replacement synchronously.
+   */
+  if (config_store->custom_write_file)
+    {
+      meta_monitor_config_store_save_sync (config_store);
+      return;
+    }
+
   config_store->save_cancellable = g_cancellable_new ();
 
   buffer = generate_config_xml (config_store);
@@ -1268,31 +1404,6 @@ meta_monitor_config_store_save (MetaMonitorConfigStore *config_store)
     .buffer = buffer
   };
 
-  /*
-   * Custom write file is only ever used by the test suite, and the test suite
-   * will want to have be able to read back the content immediately, so for
-   * custom write files, do the content replacement synchronously.
-   */
-  if (config_store->custom_write_file)
-    {
-      GError *error = NULL;
-
-      if (!g_file_replace_contents (config_store->custom_write_file,
-                                    buffer->str, buffer->len,
-                                    NULL,
-                                    FALSE,
-                                    G_FILE_CREATE_REPLACE_DESTINATION,
-                                    NULL,
-                                    NULL,
-                                    &error))
-        {
-          g_warning ("Saving monitor configuration failed: %s\n",
-                     error->message);
-          g_error_free (error);
-        }
-      return;
-    }
-
   g_file_replace_contents_async (config_store->user_file,
                                  buffer->str, buffer->len,
                                  NULL,
@@ -1407,6 +1518,8 @@ meta_monitor_config_store_constructed (GObject *object)
     }
 
   g_free (user_file_path);
+
+  G_OBJECT_CLASS (meta_monitor_config_store_parent_class)->constructed (object);
 }
 
 static void
@@ -1414,6 +1527,14 @@ meta_monitor_config_store_dispose (GObject *object)
 {
   MetaMonitorConfigStore *config_store = META_MONITOR_CONFIG_STORE (object);
 
+  if (config_store->save_cancellable)
+    {
+      g_cancellable_cancel (config_store->save_cancellable);
+      g_clear_object (&config_store->save_cancellable);
+
+      meta_monitor_config_store_save_sync (config_store);
+    }
+
   g_clear_pointer (&config_store->configs, g_hash_table_destroy);
 
   g_clear_object (&config_store->user_file);
