@@ -36,16 +36,30 @@
 #include "util-private.h"
 #include <meta/errors.h>
 #include "edid.h"
+#include "backends/meta-crtc.h"
 #include "backends/meta-logical-monitor.h"
 #include "backends/meta-monitor.h"
 #include "backends/meta-monitor-config-manager.h"
 #include "backends/meta-orientation-manager.h"
+#include "backends/meta-output.h"
 #include "backends/x11/meta-monitor-manager-xrandr.h"
 #include "meta-backend-private.h"
 
 #define DEFAULT_DISPLAY_CONFIGURATION_TIMEOUT 20
 
+enum
+{
+  PROP_0,
+
+  PROP_BACKEND,
+
+  PROP_LAST
+};
+
+static GParamSpec *obj_props[PROP_LAST];
+
 enum {
+  MONITORS_CHANGED_INTERNAL,
   CONFIRM_DISPLAY_CHANGE,
   SIGNALS_LAST
 };
@@ -75,6 +89,12 @@ static gboolean
 meta_monitor_manager_is_config_complete (MetaMonitorManager *manager,
                                          MetaMonitorsConfig *config);
 
+MetaBackend *
+meta_monitor_manager_get_backend (MetaMonitorManager *manager)
+{
+  return manager->backend;
+}
+
 static void
 meta_monitor_manager_init (MetaMonitorManager *manager)
 {
@@ -446,13 +466,13 @@ meta_monitor_manager_apply_monitors_config (MetaMonitorManager      *manager,
 gboolean
 meta_monitor_manager_has_hotplug_mode_update (MetaMonitorManager *manager)
 {
-  unsigned int i;
+  GList *l;
 
-  for (i = 0; i < manager->n_outputs; i++)
+  for (l = manager->gpus; l; l = l->next)
     {
-      MetaOutput *output = &manager->outputs[i];
+      MetaGpu *gpu = l->data;
 
-      if (output->hotplug_mode_update)
+      if (meta_gpu_has_hotplug_mode_update (gpu))
         return TRUE;
     }
 
@@ -674,13 +694,27 @@ experimental_features_changed (MetaSettings           *settings,
   meta_settings_update_ui_scaling_factor (settings);
 }
 
+void
+meta_monitor_manager_setup (MetaMonitorManager *manager)
+{
+  manager->in_init = TRUE;
+
+  manager->config_manager = meta_monitor_config_manager_new (manager);
+
+  meta_monitor_manager_read_current_state (manager);
+
+  meta_monitor_manager_ensure_initial_config (manager);
+
+  manager->in_init = FALSE;
+}
+
 static void
 meta_monitor_manager_constructed (GObject *object)
 {
   MetaMonitorManager *manager = META_MONITOR_MANAGER (object);
   MetaMonitorManagerClass *manager_class =
     META_MONITOR_MANAGER_GET_CLASS (manager);
-  MetaBackend *backend = meta_get_backend ();
+  MetaBackend *backend = manager->backend;
   MetaSettings *settings = meta_backend_get_settings (backend);
 
   manager->experimental_features_changed_handler_id =
@@ -705,90 +739,8 @@ meta_monitor_manager_constructed (GObject *object)
                            manager, 0);
 
   manager->current_switch_config = META_MONITOR_SWITCH_CONFIG_UNKNOWN;
-  manager->in_init = TRUE;
-
-  manager->config_manager = meta_monitor_config_manager_new (manager);
-
-  meta_monitor_manager_read_current_state (manager);
-
-  meta_monitor_manager_ensure_initial_config (manager);
 
   initialize_dbus_interface (manager);
-
-  manager->in_init = FALSE;
-}
-
-void
-meta_monitor_manager_clear_output (MetaOutput *output)
-{
-  g_free (output->name);
-  g_free (output->vendor);
-  g_free (output->product);
-  g_free (output->serial);
-  g_free (output->modes);
-  g_free (output->possible_crtcs);
-  g_free (output->possible_clones);
-
-  if (output->driver_notify)
-    output->driver_notify (output);
-
-  memset (output, 0, sizeof (*output));
-}
-
-static void
-meta_monitor_manager_free_output_array (MetaOutput *old_outputs,
-                                        int         n_old_outputs)
-{
-  int i;
-
-  for (i = 0; i < n_old_outputs; i++)
-    meta_monitor_manager_clear_output (&old_outputs[i]);
-
-  g_free (old_outputs);
-}
-
-void
-meta_monitor_manager_clear_mode (MetaCrtcMode *mode)
-{
-  g_free (mode->name);
-
-  if (mode->driver_notify)
-    mode->driver_notify (mode);
-
-  memset (mode, 0, sizeof (*mode));
-}
-
-static void
-meta_monitor_manager_free_mode_array (MetaCrtcMode *old_modes,
-                                      int           n_old_modes)
-{
-  int i;
-
-  for (i = 0; i < n_old_modes; i++)
-    meta_monitor_manager_clear_mode (&old_modes[i]);
-
-  g_free (old_modes);
-}
-
-void
-meta_monitor_manager_clear_crtc (MetaCrtc *crtc)
-{
-  if (crtc->driver_notify)
-    crtc->driver_notify (crtc);
-
-  memset (crtc, 0, sizeof (*crtc));
-}
-
-static void
-meta_monitor_manager_free_crtc_array (MetaCrtc *old_crtcs,
-                                      int       n_old_crtcs)
-{
-  int i;
-
-  for (i = 0; i < n_old_crtcs; i++)
-    meta_monitor_manager_clear_crtc (&old_crtcs[i]);
-
-  g_free (old_crtcs);
 }
 
 static void
@@ -796,12 +748,10 @@ meta_monitor_manager_finalize (GObject *object)
 {
   MetaMonitorManager *manager = META_MONITOR_MANAGER (object);
 
-  meta_monitor_manager_free_output_array (manager->outputs, manager->n_outputs);
-  meta_monitor_manager_free_mode_array (manager->modes, manager->n_modes);
-  meta_monitor_manager_free_crtc_array (manager->crtcs, manager->n_crtcs);
+  g_list_free_full (manager->gpus, g_object_unref);
   g_list_free_full (manager->logical_monitors, g_object_unref);
 
-  g_signal_handler_disconnect (meta_get_backend (),
+  g_signal_handler_disconnect (manager->backend,
                                manager->experimental_features_changed_handler_id);
 
   G_OBJECT_CLASS (meta_monitor_manager_parent_class)->finalize (object);
@@ -838,6 +788,42 @@ meta_monitor_manager_real_get_edid_file (MetaMonitorManager *manager,
   return NULL;
 }
 
+static void
+meta_monitor_manager_set_property (GObject      *object,
+                                   guint         prop_id,
+                                   const GValue *value,
+                                   GParamSpec   *pspec)
+{
+  MetaMonitorManager *manager = META_MONITOR_MANAGER (object);
+
+  switch (prop_id)
+    {
+    case PROP_BACKEND:
+      manager->backend = g_value_get_object (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    }
+}
+
+static void
+meta_monitor_manager_get_property (GObject    *object,
+                                   guint       prop_id,
+                                   GValue     *value,
+                                   GParamSpec *pspec)
+{
+  MetaMonitorManager *manager = META_MONITOR_MANAGER (object);
+
+  switch (prop_id)
+    {
+    case PROP_BACKEND:
+      g_value_set_object (value, manager->backend);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    }
+}
+
 static void
 meta_monitor_manager_class_init (MetaMonitorManagerClass *klass)
 {
@@ -846,11 +832,21 @@ meta_monitor_manager_class_init (MetaMonitorManagerClass *klass)
   object_class->constructed = meta_monitor_manager_constructed;
   object_class->dispose = meta_monitor_manager_dispose;
   object_class->finalize = meta_monitor_manager_finalize;
+  object_class->get_property = meta_monitor_manager_get_property;
+  object_class->set_property = meta_monitor_manager_set_property;
 
   klass->get_edid_file = meta_monitor_manager_real_get_edid_file;
   klass->read_edid = meta_monitor_manager_real_read_edid;
   klass->is_lid_closed = meta_monitor_manager_real_is_lid_closed;
 
+  signals[MONITORS_CHANGED_INTERNAL] =
+    g_signal_new ("monitors-changed-internal",
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 0);
+
   signals[CONFIRM_DISPLAY_CHANGE] =
     g_signal_new ("confirm-display-change",
 		  G_TYPE_FROM_CLASS (object_class),
@@ -858,6 +854,16 @@ meta_monitor_manager_class_init (MetaMonitorManagerClass *klass)
 		  0,
                   NULL, NULL, NULL,
 		  G_TYPE_NONE, 0);
+
+  obj_props[PROP_BACKEND] =
+    g_param_spec_object ("backend",
+                         "backend",
+                         "MetaBackend",
+                         META_TYPE_BACKEND,
+                         G_PARAM_READWRITE |
+                         G_PARAM_CONSTRUCT_ONLY |
+                         G_PARAM_STATIC_STRINGS);
+  g_object_class_install_properties (object_class, PROP_LAST, obj_props);
 }
 
 static const double known_diagonals[] = {
@@ -958,31 +964,61 @@ get_connector_type_name (MetaConnectorType connector_type)
     }
 }
 
+static GList *
+combine_gpu_lists (MetaMonitorManager    *manager,
+                   GList              * (*list_getter) (MetaGpu *gpu))
+{
+  GList *list = NULL;
+  GList *l;
+
+  for (l = manager->gpus; l; l = l->next)
+    {
+      MetaGpu *gpu = l->data;
+
+      list = g_list_concat (list, g_list_copy (list_getter (gpu)));
+    }
+
+  return list;
+}
+
 static gboolean
 meta_monitor_manager_handle_get_resources (MetaDBusDisplayConfig *skeleton,
                                            GDBusMethodInvocation *invocation)
 {
   MetaMonitorManager *manager = META_MONITOR_MANAGER (skeleton);
   MetaMonitorManagerClass *manager_class = META_MONITOR_MANAGER_GET_CLASS (skeleton);
+  GList *combined_modes;
+  GList *combined_outputs;
+  GList *combined_crtcs;
   GVariantBuilder crtc_builder, output_builder, mode_builder;
+  GList *l;
   unsigned int i, j;
   int max_screen_width;
   int max_screen_height;
 
+  combined_modes = combine_gpu_lists (manager, meta_gpu_get_modes);
+  combined_outputs = combine_gpu_lists (manager, meta_gpu_get_outputs);
+  combined_crtcs = combine_gpu_lists (manager, meta_gpu_get_crtcs);
+
   g_variant_builder_init (&crtc_builder, G_VARIANT_TYPE ("a(uxiiiiiuaua{sv})"));
   g_variant_builder_init (&output_builder, G_VARIANT_TYPE ("a(uxiausauaua{sv})"));
   g_variant_builder_init (&mode_builder, G_VARIANT_TYPE ("a(uxuudu)"));
 
-  for (i = 0; i < manager->n_crtcs; i++)
+  for (l = combined_crtcs, i = 0; l; l = l->next, i++)
     {
-      MetaCrtc *crtc = &manager->crtcs[i];
+      MetaCrtc *crtc = l->data;
       GVariantBuilder transforms;
+      int current_mode_index;
 
       g_variant_builder_init (&transforms, G_VARIANT_TYPE ("au"));
       for (j = 0; j <= META_MONITOR_TRANSFORM_FLIPPED_270; j++)
         if (crtc->all_transforms & (1 << j))
           g_variant_builder_add (&transforms, "u", j);
 
+      if (crtc->current_mode)
+        current_mode_index = g_list_index (combined_modes, crtc->current_mode);
+      else
+        current_mode_index = -1;
       g_variant_builder_add (&crtc_builder, "(uxiiiiiuaua{sv})",
                              i, /* ID */
                              (gint64)crtc->crtc_id,
@@ -990,33 +1026,48 @@ meta_monitor_manager_handle_get_resources (MetaDBusDisplayConfig *skeleton,
                              (int)crtc->rect.y,
                              (int)crtc->rect.width,
                              (int)crtc->rect.height,
-                             (int)(crtc->current_mode ? crtc->current_mode - manager->modes : -1),
+                             current_mode_index,
                              (guint32)crtc->transform,
                              &transforms,
                              NULL /* properties */);
     }
 
-  for (i = 0; i < manager->n_outputs; i++)
+  for (l = combined_outputs; l; l = l->next)
     {
-      MetaOutput *output = &manager->outputs[i];
+      MetaOutput *output = l->data;
       GVariantBuilder crtcs, modes, clones, properties;
       GBytes *edid;
       char *edid_file;
+      int crtc_index;
 
       g_variant_builder_init (&crtcs, G_VARIANT_TYPE ("au"));
       for (j = 0; j < output->n_possible_crtcs; j++)
-        g_variant_builder_add (&crtcs, "u",
-                               (unsigned)(output->possible_crtcs[j] - manager->crtcs));
+        {
+          MetaCrtc *possible_crtc = output->possible_crtcs[j];
+          unsigned possible_crtc_index;
+
+          possible_crtc_index = g_list_index (combined_crtcs, possible_crtc);
+          g_variant_builder_add (&crtcs, "u", possible_crtc_index);
+        }
 
       g_variant_builder_init (&modes, G_VARIANT_TYPE ("au"));
       for (j = 0; j < output->n_modes; j++)
-        g_variant_builder_add (&modes, "u",
-                               (unsigned)(output->modes[j] - manager->modes));
+        {
+          unsigned mode_index;
+
+          mode_index = g_list_index (combined_modes, output->modes[j]);
+          g_variant_builder_add (&modes, "u", mode_index);
+        }
 
       g_variant_builder_init (&clones, G_VARIANT_TYPE ("au"));
       for (j = 0; j < output->n_possible_clones; j++)
-        g_variant_builder_add (&clones, "u",
-                               (unsigned)(output->possible_clones[j] - manager->outputs));
+        {
+          unsigned int possible_clone_index;
+
+          possible_clone_index = g_list_index (combined_outputs,
+                                               output->possible_clones[j]);
+          g_variant_builder_add (&clones, "u", possible_clone_index);
+        }
 
       g_variant_builder_init (&properties, G_VARIANT_TYPE ("a{sv}"));
       g_variant_builder_add (&properties, "{sv}", "vendor",
@@ -1080,10 +1131,12 @@ meta_monitor_manager_handle_get_resources (MetaDBusDisplayConfig *skeleton,
                                                 output->tile_info.tile_h));
         }
 
+      crtc_index = output->crtc ? g_list_index (combined_crtcs, output->crtc)
+                                : -1;
       g_variant_builder_add (&output_builder, "(uxiausauaua{sv})",
                              i, /* ID */
                              (gint64)output->winsys_id,
-                             (int)(output->crtc ? output->crtc - manager->crtcs : -1),
+                             crtc_index,
                              &crtcs,
                              output->name,
                              &modes,
@@ -1091,9 +1144,9 @@ meta_monitor_manager_handle_get_resources (MetaDBusDisplayConfig *skeleton,
                              &properties);
     }
 
-  for (i = 0; i < manager->n_modes; i++)
+  for (l = combined_modes, i = 0; l; l = l->next, i++)
     {
-      MetaCrtcMode *mode = &manager->modes[i];
+      MetaCrtcMode *mode = l->data;
 
       g_variant_builder_add (&mode_builder, "(uxuudu)",
                              i, /* ID */
@@ -1121,6 +1174,11 @@ meta_monitor_manager_handle_get_resources (MetaDBusDisplayConfig *skeleton,
                                                    g_variant_builder_end (&mode_builder),
                                                    max_screen_width,
                                                    max_screen_height);
+
+  g_list_free (combined_modes);
+  g_list_free (combined_outputs);
+  g_list_free (combined_crtcs);
+
   return TRUE;
 }
 
@@ -1212,12 +1270,14 @@ meta_monitor_manager_handle_get_current_state (MetaDBusDisplayConfig *skeleton,
                                                GDBusMethodInvocation *invocation)
 {
   MetaMonitorManager *manager = META_MONITOR_MANAGER (skeleton);
+  MetaSettings *settings = meta_backend_get_settings (manager->backend);
   GVariantBuilder monitors_builder;
   GVariantBuilder logical_monitors_builder;
   GVariantBuilder properties_builder;
   GList *l;
   int i;
   MetaMonitorManagerCapability capabilities;
+  int ui_scaling_factor;
   int max_screen_width, max_screen_height;
 
   g_variant_builder_init (&monitors_builder,
@@ -1395,6 +1455,11 @@ meta_monitor_manager_handle_get_current_state (MetaDBusDisplayConfig *skeleton,
                              g_variant_new_boolean (TRUE));
     }
 
+  ui_scaling_factor = meta_settings_get_ui_scaling_factor (settings);
+  g_variant_builder_add (&properties_builder, "{sv}",
+                         "legacy-ui-scaling-factor",
+                         g_variant_new_int32 (ui_scaling_factor));
+
   if (meta_monitor_manager_get_max_screen_size (manager,
                                                 &max_screen_width,
                                                 &max_screen_height))
@@ -2020,6 +2085,7 @@ meta_monitor_manager_handle_change_backlight  (MetaDBusDisplayConfig *skeleton,
                                                gint                   value)
 {
   MetaMonitorManager *manager = META_MONITOR_MANAGER (skeleton);
+  GList *combined_outputs;
   MetaOutput *output;
 
   if (serial != manager->serial)
@@ -2030,14 +2096,18 @@ meta_monitor_manager_handle_change_backlight  (MetaDBusDisplayConfig *skeleton,
       return TRUE;
     }
 
-  if (output_index >= manager->n_outputs)
+  combined_outputs = combine_gpu_lists (manager, meta_gpu_get_outputs);
+
+  if (output_index >= g_list_length (combined_outputs))
     {
+      g_list_free (combined_outputs);
       g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
                                              G_DBUS_ERROR_INVALID_ARGS,
                                              "Invalid output id");
       return TRUE;
     }
-  output = &manager->outputs[output_index];
+  output = g_list_nth_data (combined_outputs, output_index);
+  g_list_free (combined_outputs);
 
   if (value < 0 || value > 100)
     {
@@ -2070,6 +2140,7 @@ meta_monitor_manager_handle_get_crtc_gamma  (MetaDBusDisplayConfig *skeleton,
 {
   MetaMonitorManager *manager = META_MONITOR_MANAGER (skeleton);
   MetaMonitorManagerClass *klass;
+  GList *combined_crtcs;
   MetaCrtc *crtc;
   gsize size;
   unsigned short *red;
@@ -2086,14 +2157,18 @@ meta_monitor_manager_handle_get_crtc_gamma  (MetaDBusDisplayConfig *skeleton,
       return TRUE;
     }
 
-  if (crtc_id >= manager->n_crtcs)
+  combined_crtcs = combine_gpu_lists (manager, meta_gpu_get_crtcs);
+  if (crtc_id >= g_list_length (combined_crtcs))
     {
+      g_list_free (combined_crtcs);
       g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
                                              G_DBUS_ERROR_INVALID_ARGS,
                                              "Invalid crtc id");
       return TRUE;
     }
-  crtc = &manager->crtcs[crtc_id];
+
+  crtc = g_list_nth_data (combined_crtcs, crtc_id);
+  g_list_free (combined_crtcs);
 
   klass = META_MONITOR_MANAGER_GET_CLASS (manager);
   if (klass->get_crtc_gamma)
@@ -2133,6 +2208,7 @@ meta_monitor_manager_handle_set_crtc_gamma  (MetaDBusDisplayConfig *skeleton,
 {
   MetaMonitorManager *manager = META_MONITOR_MANAGER (skeleton);
   MetaMonitorManagerClass *klass;
+  GList *combined_crtcs;
   MetaCrtc *crtc;
   gsize size, dummy;
   unsigned short *red;
@@ -2148,14 +2224,19 @@ meta_monitor_manager_handle_set_crtc_gamma  (MetaDBusDisplayConfig *skeleton,
       return TRUE;
     }
 
-  if (crtc_id >= manager->n_crtcs)
+  combined_crtcs = combine_gpu_lists (manager, meta_gpu_get_crtcs);
+
+  if (crtc_id >= g_list_length (combined_crtcs))
     {
+      g_list_free (combined_crtcs);
       g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
                                              G_DBUS_ERROR_INVALID_ARGS,
                                              "Invalid crtc id");
       return TRUE;
     }
-  crtc = &manager->crtcs[crtc_id];
+
+  crtc = g_list_nth_data (combined_crtcs, crtc_id);
+  g_list_free (combined_crtcs);
 
   red_bytes = g_variant_get_data_as_bytes (red_v);
   green_bytes = g_variant_get_data_as_bytes (green_v);
@@ -2264,7 +2345,7 @@ MetaLogicalMonitor *
 meta_monitor_manager_get_logical_monitor_from_number (MetaMonitorManager *manager,
                                                       int                 number)
 {
-  g_assert ((unsigned int) number < g_list_length (manager->logical_monitors));
+  g_return_val_if_fail ((unsigned int) number < g_list_length (manager->logical_monitors), NULL);
 
   return g_list_nth (manager->logical_monitors, number)->data;
 }
@@ -2425,38 +2506,17 @@ meta_monitor_manager_get_monitors (MetaMonitorManager *manager)
   return manager->monitors;
 }
 
-MetaOutput *
-meta_monitor_manager_get_outputs (MetaMonitorManager *manager,
-                                  unsigned int       *n_outputs)
+void
+meta_monitor_manager_add_gpu (MetaMonitorManager *manager,
+                              MetaGpu            *gpu)
 {
-  *n_outputs = manager->n_outputs;
-  return manager->outputs;
+  manager->gpus = g_list_append (manager->gpus, gpu);
 }
 
-void
-meta_monitor_manager_get_resources (MetaMonitorManager  *manager,
-                                    MetaCrtcMode       **modes,
-                                    unsigned int        *n_modes,
-                                    MetaCrtc           **crtcs,
-                                    unsigned int        *n_crtcs,
-                                    MetaOutput         **outputs,
-                                    unsigned int        *n_outputs)
+GList *
+meta_monitor_manager_get_gpus (MetaMonitorManager *manager)
 {
-  if (modes)
-    {
-      *modes = manager->modes;
-      *n_modes = manager->n_modes;
-    }
-  if (crtcs)
-    {
-      *crtcs = manager->crtcs;
-      *n_crtcs = manager->n_crtcs;
-    }
-  if (outputs)
-    {
-      *outputs = manager->outputs;
-      *n_outputs = manager->n_outputs;
-    }
+  return manager->gpus;
 }
 
 void
@@ -2471,7 +2531,7 @@ meta_monitor_manager_get_screen_size (MetaMonitorManager *manager,
 static void
 rebuild_monitors (MetaMonitorManager *manager)
 {
-  unsigned int i;
+  GList *l;
 
   if (manager->monitors)
     {
@@ -2479,28 +2539,34 @@ rebuild_monitors (MetaMonitorManager *manager)
       manager->monitors = NULL;
     }
 
-  for (i = 0; i < manager->n_outputs; i++)
+  for (l = manager->gpus; l; l = l->next)
     {
-      MetaOutput *output = &manager->outputs[i];
+      MetaGpu *gpu = l->data;
+      GList *k;
 
-      if (output->tile_info.group_id)
+      for (k = meta_gpu_get_outputs (gpu); k; k = k->next)
         {
-          if (is_main_tiled_monitor_output (output))
+          MetaOutput *output = k->data;
+
+          if (output->tile_info.group_id)
             {
-              MetaMonitorTiled *monitor_tiled;
+              if (is_main_tiled_monitor_output (output))
+                {
+                  MetaMonitorTiled *monitor_tiled;
 
-              monitor_tiled = meta_monitor_tiled_new (manager, output);
-              manager->monitors = g_list_append (manager->monitors,
-                                                 monitor_tiled);
+                  monitor_tiled = meta_monitor_tiled_new (gpu, output);
+                  manager->monitors = g_list_append (manager->monitors,
+                                                     monitor_tiled);
+                }
             }
-        }
-      else
-        {
-          MetaMonitorNormal *monitor_normal;
+          else
+            {
+              MetaMonitorNormal *monitor_normal;
 
-          monitor_normal = meta_monitor_normal_new (manager, output);
-          manager->monitors = g_list_append (manager->monitors,
-                                             monitor_normal);
+              monitor_normal = meta_monitor_normal_new (gpu, output);
+              manager->monitors = g_list_append (manager->monitors,
+                                                 monitor_normal);
+            }
         }
     }
 }
@@ -2541,39 +2607,33 @@ meta_monitor_manager_is_transform_handled (MetaMonitorManager  *manager,
 void
 meta_monitor_manager_read_current_state (MetaMonitorManager *manager)
 {
-  MetaOutput *old_outputs;
-  MetaCrtc *old_crtcs;
-  MetaCrtcMode *old_modes;
-  unsigned int n_old_outputs, n_old_crtcs, n_old_modes;
-
-  /* Some implementations of read_current use the existing information
-   * we have available, so don't free the old configuration until after
-   * read_current finishes. */
-  old_outputs = manager->outputs;
-  n_old_outputs = manager->n_outputs;
-  old_crtcs = manager->crtcs;
-  n_old_crtcs = manager->n_crtcs;
-  old_modes = manager->modes;
-  n_old_modes = manager->n_modes;
+  GList *l;
 
   manager->serial++;
-  META_MONITOR_MANAGER_GET_CLASS (manager)->read_current (manager);
+
+  for (l = manager->gpus; l; l = l->next)
+    {
+      MetaGpu *gpu = l->data;
+      GError *error = NULL;
+
+      if (!meta_gpu_read_current (gpu, &error))
+        {
+          g_warning ("Failed to read current KMS state: %s", error->message);
+          g_clear_error (&error);
+        }
+    }
 
   rebuild_monitors (manager);
-
-  meta_monitor_manager_free_output_array (old_outputs, n_old_outputs);
-  meta_monitor_manager_free_mode_array (old_modes, n_old_modes);
-  meta_monitor_manager_free_crtc_array (old_crtcs, n_old_crtcs);
 }
 
 static void
 meta_monitor_manager_notify_monitors_changed (MetaMonitorManager *manager)
 {
-  MetaBackend *backend = meta_get_backend ();
-
   manager->current_switch_config = META_MONITOR_SWITCH_CONFIG_UNKNOWN;
 
-  meta_backend_monitors_changed (backend);
+  meta_backend_monitors_changed (manager->backend);
+
+  g_signal_emit (manager, signals[MONITORS_CHANGED_INTERNAL], 0);
   g_signal_emit_by_name (manager, "monitors-changed");
 }
 
@@ -2816,40 +2876,6 @@ meta_monitor_manager_get_monitor_matrix (MetaMonitorManager *manager,
   return TRUE;
 }
 
-/**
- * meta_monitor_manager_get_monitor_for_output:
- * @manager: A #MetaMonitorManager
- * @id: A valid #MetaOutput id
- *
- * Returns: The monitor index or -1 if @id isn't valid or the output
- * isn't associated with a logical monitor.
- */
-gint
-meta_monitor_manager_get_monitor_for_output (MetaMonitorManager *manager,
-                                             guint               id)
-{
-  MetaOutput *output;
-  GList *l;
-
-  g_return_val_if_fail (META_IS_MONITOR_MANAGER (manager), -1);
-  g_return_val_if_fail (id < manager->n_outputs, -1);
-
-  output = &manager->outputs[id];
-  if (!output || !output->crtc)
-    return -1;
-
-  for (l = manager->logical_monitors; l; l = l->next)
-    {
-      MetaLogicalMonitor *logical_monitor = l->data;
-
-      if (meta_rectangle_contains_rect (&logical_monitor->rect,
-                                        &output->crtc->rect))
-        return logical_monitor->number;
-    }
-
-  return -1;
-}
-
 /**
  * meta_monitor_manager_get_monitor_for_connector:
  * @manager: A #MetaMonitorManager
