@@ -24,8 +24,11 @@
 #include "backends/meta-monitor.h"
 
 #include "backends/meta-backend-private.h"
+#include "backends/meta-crtc.h"
+#include "backends/meta-gpu.h"
 #include "backends/meta-monitor-manager-private.h"
 #include "backends/meta-settings-private.h"
+#include "backends/meta-output.h"
 
 #define SCALE_FACTORS_PER_INTEGER 4
 #define MINIMUM_SCALE_FACTOR 1.0f
@@ -34,6 +37,8 @@
 #define MINIMUM_LOGICAL_HEIGHT 600
 #define MAXIMUM_REFRESH_RATE_DIFF 0.001
 
+#define HANDLED_CRTC_MODE_FLAGS (META_CRTC_MODE_FLAG_INTERLACE)
+
 typedef struct _MetaMonitorMode
 {
   char *id;
@@ -50,7 +55,7 @@ typedef struct _MetaMonitorModeTiled
 
 typedef struct _MetaMonitorPrivate
 {
-  MetaMonitorManager *monitor_manager;
+  MetaGpu *gpu;
 
   GList *outputs;
   GList *modes;
@@ -175,6 +180,14 @@ meta_monitor_generate_spec (MetaMonitor *monitor)
   priv->spec = monitor_spec;
 }
 
+MetaGpu *
+meta_monitor_get_gpu (MetaMonitor *monitor)
+{
+  MetaMonitorPrivate *priv = meta_monitor_get_instance_private (monitor);
+
+  return priv->gpu;
+}
+
 GList *
 meta_monitor_get_outputs (MetaMonitor *monitor)
 {
@@ -392,22 +405,16 @@ generate_mode_id (MetaMonitorModeSpec *monitor_mode_spec)
 
 static gboolean
 meta_monitor_add_mode (MetaMonitor     *monitor,
-                       MetaMonitorMode *monitor_mode,
-                       gboolean         replace)
+                       MetaMonitorMode *monitor_mode)
 {
   MetaMonitorPrivate *priv = meta_monitor_get_instance_private (monitor);
-  MetaMonitorMode *existing_mode;
 
-  existing_mode = g_hash_table_lookup (priv->mode_ids,
-                                       meta_monitor_mode_get_id (monitor_mode));
-  if (existing_mode && !replace)
+  if (g_hash_table_lookup (priv->mode_ids,
+                           meta_monitor_mode_get_id (monitor_mode)))
     return FALSE;
 
-  if (existing_mode)
-    priv->modes = g_list_remove (priv->modes, existing_mode);
-
   priv->modes = g_list_append (priv->modes, monitor_mode);
-  g_hash_table_replace (priv->mode_ids, monitor_mode->id, monitor_mode);
+  g_hash_table_insert (priv->mode_ids, monitor_mode->id, monitor_mode);
 
   return TRUE;
 }
@@ -419,17 +426,13 @@ meta_monitor_normal_generate_modes (MetaMonitorNormal *monitor_normal)
   MetaMonitorPrivate *monitor_priv =
     meta_monitor_get_instance_private (monitor);
   MetaOutput *output;
-  MetaCrtcModeFlag preferred_mode_flags;
   unsigned int i;
 
   output = meta_monitor_get_main_output (monitor);
-  preferred_mode_flags = output->preferred_mode->flags;
-
   for (i = 0; i < output->n_modes; i++)
     {
       MetaCrtcMode *crtc_mode = output->modes[i];
       MetaMonitorMode *mode;
-      gboolean replace;
 
       mode = g_new0 (MetaMonitorMode, 1);
       mode->spec = (MetaMonitorModeSpec) {
@@ -445,32 +448,19 @@ meta_monitor_normal_generate_modes (MetaMonitorNormal *monitor_normal)
         .crtc_mode = crtc_mode
       };
 
-      /*
-       * We don't distinguish between all available mode flags, just the ones
-       * that are configurable. We still need to pick some mode though, so
-       * prefer ones that has the same set of flags as the preferred mode;
-       * otherwise take the first one in the list. This guarantees that the
-       * preferred mode is always added.
-       */
-      replace = crtc_mode->flags == preferred_mode_flags;
-
-      if (!meta_monitor_add_mode (monitor, mode, replace))
-        {
-          g_assert (crtc_mode != output->preferred_mode);
-          meta_monitor_mode_free (mode);
-          continue;
-        }
-
       if (crtc_mode == output->preferred_mode)
         monitor_priv->preferred_mode = mode;
       if (output->crtc && crtc_mode == output->crtc->current_mode)
         monitor_priv->current_mode = mode;
+
+      if (!meta_monitor_add_mode (monitor, mode))
+        meta_monitor_mode_free (mode);
     }
 }
 
 MetaMonitorNormal *
-meta_monitor_normal_new (MetaMonitorManager *monitor_manager,
-                         MetaOutput         *output)
+meta_monitor_normal_new (MetaGpu    *gpu,
+                         MetaOutput *output)
 {
   MetaMonitorNormal *monitor_normal;
   MetaMonitor *monitor;
@@ -480,7 +470,7 @@ meta_monitor_normal_new (MetaMonitorManager *monitor_manager,
   monitor = META_MONITOR (monitor_normal);
   monitor_priv = meta_monitor_get_instance_private (monitor);
 
-  monitor_priv->monitor_manager = monitor_manager;
+  monitor_priv->gpu = gpu;
 
   monitor_priv->outputs = g_list_append (NULL, output);
   monitor_priv->winsys_id = output->winsys_id;
@@ -576,16 +566,18 @@ meta_monitor_get_suggested_position (MetaMonitor *monitor,
 }
 
 static void
-add_tiled_monitor_outputs (MetaMonitorManager *monitor_manager,
-                           MetaMonitorTiled   *monitor_tiled)
+add_tiled_monitor_outputs (MetaGpu          *gpu,
+                           MetaMonitorTiled *monitor_tiled)
 {
   MetaMonitorPrivate *monitor_priv =
     meta_monitor_get_instance_private (META_MONITOR (monitor_tiled));
-  unsigned int i;
+  GList *outputs;
+  GList *l;
 
-  for (i = 0; i < monitor_manager->n_outputs; i++)
+  outputs = meta_gpu_get_outputs (gpu);
+  for (l = outputs; l; l = l->next)
     {
-      MetaOutput *output = &monitor_manager->outputs[i];
+      MetaOutput *output = l->data;
 
       if (output->tile_info.group_id != monitor_tiled->tile_group_id)
         continue;
@@ -846,7 +838,7 @@ generate_tiled_monitor_modes (MetaMonitorTiled *monitor_tiled)
 
       tiled_modes = g_list_remove_link (tiled_modes, l);
 
-      if (!meta_monitor_add_mode (monitor, mode, FALSE))
+      if (!meta_monitor_add_mode (monitor, mode))
         {
           meta_monitor_mode_free (mode);
           continue;
@@ -988,7 +980,7 @@ generate_untiled_monitor_modes (MetaMonitorTiled *monitor_tiled)
       if (!mode)
         continue;
 
-      if (!meta_monitor_add_mode (monitor, mode, FALSE))
+      if (!meta_monitor_add_mode (monitor, mode))
         {
           meta_monitor_mode_free (mode);
           continue;
@@ -1101,9 +1093,10 @@ meta_monitor_tiled_generate_modes (MetaMonitorTiled *monitor_tiled)
 }
 
 MetaMonitorTiled *
-meta_monitor_tiled_new (MetaMonitorManager *monitor_manager,
-                        MetaOutput         *output)
+meta_monitor_tiled_new (MetaGpu    *gpu,
+                        MetaOutput *output)
 {
+  MetaMonitorManager *monitor_manager;
   MetaMonitorTiled *monitor_tiled;
   MetaMonitor *monitor;
   MetaMonitorPrivate *monitor_priv;
@@ -1112,18 +1105,19 @@ meta_monitor_tiled_new (MetaMonitorManager *monitor_manager,
   monitor = META_MONITOR (monitor_tiled);
   monitor_priv = meta_monitor_get_instance_private (monitor);
 
-  monitor_priv->monitor_manager = monitor_manager;
+  monitor_priv->gpu = gpu;
 
   monitor_tiled->tile_group_id = output->tile_info.group_id;
   monitor_priv->winsys_id = output->winsys_id;
 
   monitor_tiled->origin_output = output;
-  add_tiled_monitor_outputs (monitor_manager, monitor_tiled);
+  add_tiled_monitor_outputs (gpu, monitor_tiled);
 
   monitor_tiled->main_output = find_untiled_output (monitor_tiled);
 
   meta_monitor_generate_spec (monitor);
 
+  monitor_manager = meta_gpu_get_monitor_manager (gpu);
   meta_monitor_manager_tiled_monitor_added (monitor_manager,
                                             META_MONITOR (monitor_tiled));
 
@@ -1210,8 +1204,10 @@ meta_monitor_tiled_finalize (GObject *object)
   MetaMonitor *monitor = META_MONITOR (object);
   MetaMonitorPrivate *monitor_priv =
     meta_monitor_get_instance_private (monitor);
+  MetaMonitorManager *monitor_manager;
 
-  meta_monitor_manager_tiled_monitor_removed (monitor_priv->monitor_manager,
+  monitor_manager = meta_gpu_get_monitor_manager (monitor_priv->gpu);
+  meta_monitor_manager_tiled_monitor_removed (monitor_manager,
                                               monitor);
 
   G_OBJECT_CLASS (meta_monitor_tiled_parent_class)->finalize (object);
