@@ -24,7 +24,9 @@
 
 #include <glib-object.h>
 
+#include "backends/meta-crtc.h"
 #include "backends/meta-monitor-manager-private.h"
+#include "backends/meta-output.h"
 
 typedef struct _MetaMonitorSpec
 {
@@ -48,8 +50,6 @@ typedef struct _MetaMonitorCrtcMode
   MetaCrtcMode *crtc_mode;
 } MetaMonitorCrtcMode;
 
-#define HANDLED_CRTC_MODE_FLAGS (META_CRTC_MODE_FLAG_INTERLACE)
-
 typedef gboolean (* MetaMonitorModeFunc) (MetaMonitor         *monitor,
                                           MetaMonitorMode     *mode,
                                           MetaMonitorCrtcMode *monitor_crtc_mode,
@@ -93,14 +93,16 @@ G_DECLARE_FINAL_TYPE (MetaMonitorTiled, meta_monitor_tiled,
                       META, MONITOR_TILED,
                       MetaMonitor)
 
-MetaMonitorTiled * meta_monitor_tiled_new (MetaMonitorManager *monitor_manager,
-                                           MetaOutput         *main_output);
+MetaMonitorTiled * meta_monitor_tiled_new (MetaGpu    *gpu,
+                                           MetaOutput *output);
 
-MetaMonitorNormal * meta_monitor_normal_new (MetaMonitorManager *monitor_manager,
-                                             MetaOutput         *output);
+MetaMonitorNormal * meta_monitor_normal_new (MetaGpu    *gpu,
+                                             MetaOutput *output);
 
 MetaMonitorSpec * meta_monitor_get_spec (MetaMonitor *monitor);
 
+MetaGpu * meta_monitor_get_gpu (MetaMonitor *monitor);
+
 gboolean meta_monitor_is_active (MetaMonitor *monitor);
 
 MetaOutput * meta_monitor_get_main_output (MetaMonitor *monitor);
