@@ -27,6 +27,7 @@
 
 #include "backends/meta-backend-private.h"
 #include "backends/native/meta-clutter-backend-native.h"
+#include "backends/native/meta-launcher.h"
 
 #define META_TYPE_BACKEND_NATIVE (meta_backend_native_get_type ())
 G_DECLARE_FINAL_TYPE (MetaBackendNative, meta_backend_native,
@@ -38,4 +39,6 @@ void meta_backend_native_pause (MetaBackendNative *backend_native);
 
 void meta_backend_native_resume (MetaBackendNative *backend_native);
 
+MetaLauncher * meta_backend_native_get_launcher (MetaBackendNative *native);
+
 #endif /* META_BACKEND_NATIVE_H */
