@@ -28,8 +28,13 @@
 #include <glib.h>
 #include <meta/types.h>
 
+#include "wayland/meta-wayland-types.h"
+
 void
 meta_xwayland_handle_wl_surface_id (MetaWindow *window,
                                     guint32     surface_id);
 
+gboolean
+meta_xwayland_is_xwayland_surface (MetaWaylandSurface *surface);
+
 #endif /* META_XWAYLAND_H */
